/*
  Copyright 2008 Riccardo Govoni battlehorse@gmail.com

  Licensed under the Apache License, Version 2.0 (the &quot;License&quot;);
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an &quot;AS IS&quot; BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

organizer = {};

organizer.MagicCard = function(id, power, element) {
  this.id = id;
  this.power = power;
  this.element = element;
};

organizer.models = [
  new organizer.MagicCard("1",  4, ['earth']),
  new organizer.MagicCard("2",  1, ['earth']),
  new organizer.MagicCard("3",  3, ['earth','water']),
  new organizer.MagicCard("4",  2, ['earth','water']),
  new organizer.MagicCard("5",  2, ['earth','water']),
  new organizer.MagicCard("6",  2, ['earth']),
  new organizer.MagicCard("7",  3, ['earth']),
  new organizer.MagicCard("8",  3, ['fire']),
  new organizer.MagicCard("9",  3, ['fire']),
  new organizer.MagicCard("10", 5, ['fire','earth']),
  new organizer.MagicCard("11", 3, ['fire','earth']),
  new organizer.MagicCard("12", 5, ['fire','earth']),
  new organizer.MagicCard("13", 1, ['fire','earth']),
  new organizer.MagicCard("14", 1, ['fire','air']),
  new organizer.MagicCard("15", 1, ['fire','air']),
  new organizer.MagicCard("16", 4, ['fire','air']),
  new organizer.MagicCard("17", 3, ['fire']),
  new organizer.MagicCard("18", 3, ['water']),
  new organizer.MagicCard("19", 2, ['water']),
  new organizer.MagicCard("20", 1, ['water']),
  new organizer.MagicCard("21", 2, ['water','air']),
  new organizer.MagicCard("22", 2, ['water','air']),
  new organizer.MagicCard("23", 2, ['water','air']),
  new organizer.MagicCard("24", 5, ['water','air']),
  new organizer.MagicCard("25", 3, ['water','air']),
  new organizer.MagicCard("26", 3, ['water']),
  new organizer.MagicCard("27", 4, ['water']),
  new organizer.MagicCard("28", 5, ['water']),
  new organizer.MagicCard("29", 5, ['air']),
  new organizer.MagicCard("30", 1, ['air']),
  new organizer.MagicCard("31", 1, ['air']),
  new organizer.MagicCard("32", 1, ['air']),
  new organizer.MagicCard("33", 4, ['air']),
  new organizer.MagicCard("34", 5, ['air']),
  new organizer.MagicCard("35", 5, ['air','fire']),
  new organizer.MagicCard("36", 3, ['air','fire']),
  new organizer.MagicCard("37", 1, ['air','fire']),
  new organizer.MagicCard("38", 2, ['air','fire']),
  new organizer.MagicCard("39", 4, ['air']),
  new organizer.MagicCard("40", 5, ['air'])
];

organizer.render = function(model) {
  var html = [];
  html.push("<div class='model'><div style='padding: 3px'>");
  html.push("<p style='font-size:10px'>");
  html.push("<img src='img/m" + ((parseInt(model.id, 10) % 13) + 1) + ".jpg' width='85px' height='122px' >");
  html.push("</p></div></div>");
  return $(html.join(''));
};

organizer.SelectionManager = function() {
  this.selectionMap_ = {};
};

organizer.SelectionManager.prototype.select = function(id) {
  this.selectionMap_[id] = id;
  $('#' + id).addClass('ui-selected');
};

organizer.SelectionManager.prototype.unselect = function(id) {
  delete this.selectionMap_[id];
  $('#' + id).removeClass('ui-selected');
};    

organizer.SelectionManager.prototype.isSelected = function(id) {
  return this.selectionMap_[id];
};        

organizer.SelectionManager.prototype.allSelected = function() {
  return this.selectionMap_;
};
$sel = new organizer.SelectionManager();

organizer.start = function() {
  organizer.createModels();
  
  organizer.initDragSelection();
  
  organizer.initZooming();
  
  organizer.initPanning();
  
  organizer.initShuffleButton();
};

organizer.createModels = function() {
  jQuery.map(organizer.models, function(model) {
    var rendering = organizer.render(model);

    // initial positioning in the DOM and layout
    $(rendering).css("position", "absolute")
                .css("top", 0)
                .css("left", 0)
                .attr("id", model.id);

    $("#universe").append(rendering);    
  })
};

organizer.initDragSelection = function() {
  $('.model').dblclick(function() {
    if ($sel.isSelected(this.id)) {
      $sel.unselect(this.id);
    } else {
      $sel.select(this.id);
    }
  });      
  
  $('#container').selectable({
    selected: function(ev, ui) {
      if (ui.selected.id) {
        $sel.select(ui.selected.id);
      }
    },
    unselected: function(ev, ui) {
      if (ui.unselected.id) {
        $sel.unselect(ui.unselected.id);
      }
    },
    filter: '.model'
  });
  
  $('.model').draggable({
    opacity: 0.5,
    cursor: 'move',
    zIndex: 10000,
    start: function(ev, ui) {

      // figure out all the initial positions for the selected elements
      // and store them.
      if ($sel.isSelected(ui.helper[0].id)) {
        for (id in $sel.allSelected()) {
          $('#'+id).data(
            "top0", 
            parseInt($('#'+id).css("top"),10) - 
              parseInt($(ui.helper[0]).css("top"),10));
          $('#'+id).data(
            "left0", 
            parseInt($('#'+id).css("left"),10) - 
              parseInt($(ui.helper[0]).css("left"),10));
        }
      }
    },
    drag: function(ev, ui) {
      if ($sel.isSelected(ui.helper[0].id)) {
        for (id in $sel.allSelected()) {
          if (id != ui.helper[0].id) {
            $('#' + id).css('top',
                            $('#'+id).data("top0") + ui.position.top);
            $('#' + id).css('left', 
                            $('#'+id).data("left0") + ui.position.left);              
          }
        }
      }
    },
    stop: function(ev, ui) {
      if ($sel.isSelected(ui.helper[0].id)) {
        for (id in $sel.allSelected()) {
         $('#'+id).removeData("top0");
         $('#'+id).removeData("left0");
        }
      }        
    }
  });  
};
organizer.initZooming = function() {
  $('#container').mousewheel(function(event, delta) {
   
    var containerTopOffset = parseFloat($('#universe').css('top'));
    var containerLeftOffset = parseFloat($('#universe').css('left'));
    var scale = 0.1;
    var percent = 1 + delta*scale;        
    $('.model').each(function() {
      organizer.rescaleRelative(this, 
        'top', 
          event.clientY - 
          containerTopOffset - 
          parseFloat($(this).css('height')) / 2,
        percent);
      organizer.rescaleRelative(this, 
        'left', 
          event.clientX - 
          containerLeftOffset - 
          parseFloat($(this).css('width')) / 2, 
        percent);          
      
      organizer.rescale(this, 'width', percent);
      organizer.rescale(this, 'height', percent);
      organizer.rescale(this, 'font-size', percent);
    });
    
    $('.model IMG').each(function() {
      organizer.rescale(this, 'width', percent);
      organizer.rescale(this, 'height', percent);      
    });
    event.preventDefault();
  });  
};

organizer.rescale = function(selector, property, percent) {
  var oldValue = parseFloat($(selector).css(property));
  $(selector).css(property, oldValue*percent);
};

organizer.rescaleRelative = function(selector, property, relative, percent) {
  var oldValue = parseFloat($(selector).css(property)) - relative;
  $(selector).css(property, oldValue*percent + relative);      
};

organizer.initPanning = function() {
  var dragDelta = {
    top: $('#container').offset().top +
         $('#universe').offset().top,
    left: $('#container').offset().left +
          $('#universe').offset().left,
  };

  $('#scroll-trigger').click(function() {
    $(this).hide();
    var viewport = $('#container');
    $('#scroll-overlay')
      .css('left', viewport.css('left'))
      .css('top', viewport.css('top'))
      .css('width', viewport.width())
      .css('height', viewport.height())
      .css('z-index', 99)
      .css('display', '');

    $('#scroll-overlay').draggable({
      helper: function() {
        return $("<div />");
      },
      start: function(ev, ui) {
        var offset = $('#universe').offset();
        $('#universe')
          .data("top0", offset.top)
          .data("left0", offset.left);
      },
      drag: function(ev, ui) {
        var universe = $('#universe');
        var offset = universe.offset();

        var dragTop = ui.position.top +
                      universe.data("top0") - dragDelta.top;
        var dragLeft = ui.position.left +
                       universe.data("left0") - dragDelta.left;

        $('#universe')
          .css('top', dragTop).css('bottom', -dragTop)
          .css('left', dragLeft).css('right', -dragLeft);
      },
      refreshPositions: false
    });
  });

  $('#scroll-done').click(function() {
    $('#scroll-overlay')
      .css('z-index', -1)
      .css('display', 'none');
    $('#scroll-trigger').show();
  });  
};

organizer.initShuffleButton = function() {
  $('#shuffleCards').click(function() {
    // reset panning
    $('#universe').css('top', 0).css('left', 0).css('bottom', 0).css('right', 0);

    var maxWidth = Math.round($('#container').width()*0.3) ;
    var maxHeight = Math.round($('#container').height()*0.3);

    $('.model').each(function(model) {
      var top = Math.round($('#container').height() / 3 +
                           Math.random()*maxHeight*2 - maxHeight);
      var left = Math.round($('#container').width() / 3 +
                            Math.random()*maxWidth*2 - maxWidth);

      var movement = {'top': top, 'left': left};
      $(this).animate(movement, 1000);
    });    
  })
};